@@ -1,7 +1,21 @@
 #' @include shadow_functions.R
 NULL
 
-#' @noRd
+#' (Internal) Parse constants for adaptive test assembly simulation
+#'
+#' \code{\link{getConstants}} is an internal function for
+#' parsing constants for adaptive test assembly simulation.
+#'
+#' @template parameter_constraints
+#' @template parameter_config_Shadow
+#' @param arg_data the \code{data} argument from \code{\link{Shadow}}.
+#' @param true_theta the \code{true_theta} argument from \code{\link{Shadow}}.
+#' @param max_info the maximum possible information attainable by any single item in the pool.
+#' This is used to determine the M value if M is not supplied.
+#'
+#' @returns \code{\link{getConstants}} returns a named list containing constants.
+#'
+#' @keywords internal
 getConstants <- function(constraints, config, arg_data, true_theta, max_info) {
 
   o <- list()
@@ -49,6 +63,9 @@ getConstants <- function(constraints, config, arg_data, true_theta, max_info) {
 
   o$exclude_method <- toupper(config@exclude_policy$method)
   o$exclude_M      <- config@exclude_policy$M
+  if (is.null(o$exclude_M)) {
+    o$exclude_M <- max_info * 100 + 1
+  }
 
   o$use_hand_scored <- !is.null(config@interim_theta$hand_scored_attribute)
   if (o$use_hand_scored) {
@@ -115,16 +132,44 @@ getConstants <- function(constraints, config, arg_data, true_theta, max_info) {
   o$segment_cut         <- config@exposure_control$segment_cut
   o$cut_lower           <- o$segment_cut[(1:o$n_segment)]
   o$cut_upper           <- o$segment_cut[(1:o$n_segment) + 1]
+  o$custom_first_segment_available <- !is.null(config@exposure_control$first_segment)
+  o$custom_first_segment_n_values  <- length(config@exposure_control$first_segment)
 
-  if (!length(o$max_exposure_rate) %in% c(1, o$n_segment)) {
-    stop("length(max_exposure_rate) must be 1 or n_segment")
+  o$overlap_control_method <- toupper(config@overlap_control$method)
+  if (o$overlap_control_method %in% c("ELIGIBILITY", "BIGM", "BIGM-BAYESIAN")) {
+    o$use_overlap_control <- TRUE
+  } else {
+    o$use_overlap_control <- FALSE
   }
+  o$overlap_M <- config@overlap_control$M
+  if (is.null(o$overlap_M)) {
+    o$overlap_M <- max_info * 10 + 1
+  }
+  o$max_overlap_rate <- config@overlap_control$max_overlap_rate
+
+  o$id <- constraints@id
 
   return(o)
 
 }
 
-#' @noRd
+#' (Internal) Sanitize item model names for C++ functions
+#'
+#' \code{\link{sanitizeModel}} is an internal function for
+#' sanitizing item model names to be passed onto C++ functions.
+#'
+#' @param model a vector containing item model names.
+#'
+#' @returns \code{\link{sanitizeModel}} returns a numeric vector, with \itemize{
+#'   \item{\code{item_1PL} converted to \code{1}}
+#'   \item{\code{item_2PL} converted to \code{2}}
+#'   \item{\code{item_3PL} converted to \code{3}}
+#'   \item{\code{item_PC} converted to \code{4}}
+#'   \item{\code{item_GPC} converted to \code{5}}
+#'   \item{\code{item_GR} converted to \code{6}}
+#' }
+#'
+#' @keywords internal
 sanitizeModel <- function(model) {
   model[which(model == "item_1PL")] <- 1
   model[which(model == "item_2PL")] <- 2
@@ -136,33 +181,68 @@ sanitizeModel <- function(model) {
   return(model)
 }
 
-#' @noRd
-getInfoFixedTheta <- function(item_selection, constants, item_pool, model) {
+#' (Internal) Precalculate item information for fixed-theta item selection methods
+#'
+#' \code{\link{getInfoFixedTheta}} is an internal function for
+#' calculating item information for fixed-theta item selection methods.
+#' This is done once at the start of the simulation and cached for speed gain.
+#'
+#' @param item_selection a list containing item selection config.
+#' @template parameter_simulation_constants
+#' @param item_pool an item pool object.
+#' @param model sanitized item model codes to be fed to C++ functions.
+#'
+#' @returns \code{\link{getInfoFixedTheta}} returns a named list containing item information values
+#' at designated thetas for each simulee.
+#'
+#' @keywords internal
+getInfoFixedTheta <- function(
+  item_selection, simulation_constants,
+  item_pool, model
+) {
 
-  nj <- constants$nj
-  o <- list()
+  nj <- simulation_constants$nj
+  info_fixed_theta <- NULL
 
   if (!is.null(item_selection$fixed_theta)) {
     if (length(item_selection$fixed_theta) == 1) {
-      o$info_fixed_theta <- lapply(seq_len(nj), function(j) calc_info(item_selection$fixed_theta, item_pool@ipar, item_pool@NCAT, model))
-      o$select_at_fixed_theta <- TRUE
+      info_fixed_theta <- lapply(
+        seq_len(nj), function(j) {
+          calc_info(item_selection$fixed_theta, item_pool@ipar, item_pool@NCAT, model)
+        }
+      )
     }
     if (length(item_selection$fixed_theta) == nj) {
-      o$info_fixed_theta <- lapply(seq_len(nj), function(j) calc_info(item_selection$fixed_theta[j], item_pool@ipar, item_pool@NCAT, model))
-      o$select_at_fixed_theta <- TRUE
+      info_fixed_theta <- lapply(
+        seq_len(nj), function(j) {
+          calc_info(item_selection$fixed_theta[j], item_pool@ipar, item_pool@NCAT, model)
+        }
+      )
     }
-    if (is.null(o$info_fixed_theta)) {
+    if (is.null(info_fixed_theta)) {
       stop("config@item_selection: length($fixed_theta) must be either 1 or nj")
     }
-  } else {
-    o$select_at_fixed_theta <- FALSE
   }
 
-  return(o)
+  return(info_fixed_theta)
 
 }
 
-#' @noRd
+#' (Internal) Compute item information at current theta estimate
+#'
+#' \code{\link{computeInfoAtCurrentTheta}} is an internal function for
+#' computing item information at current theta estimate.
+#'
+#' @param item_selection the \code{item_selection} slot of a \code{\linkS4class{config_Shadow}} object.
+#' @param j the examinee index.
+#' @param current_theta a named list containing current theta estimate.
+#' @param item_pool an \code{\linkS4class{item_pool}} object.
+#' @param model_code machine-sanitized item model codes for passing onto C++ functions.
+#' @param info_fixed_theta a list containing item information at prespecified theta points.
+#' @param info_grid a matrix containing item information across quadrature points.
+#' @param item_parameter_sample a list containing item parameter samples for Bayesian estimation.
+#'
+#' @keywords internal
 computeInfoAtCurrentTheta <- function(
   item_selection,
   j,
@@ -199,21 +279,32 @@ computeInfoAtCurrentTheta <- function(
       item_pool@ipar, item_pool@NCAT, model_code)[, 1]
     return(info)
   }
-  if (item_method == "FB" & info_type == "FISHER") {
+  if (item_method == "FB" && info_type == "FISHER") {
     info <- calc_info_FB(
       matrix(current_theta$posterior_sample),
       item_parameter_sample, item_pool@NCAT, model_code)[, 1]
     return(info)
   }
-  if (item_method == "FB" & info_type %in% c("MI", "MUTUAL")) {
+  if (item_method == "FB" && info_type %in% c("MI", "MUTUAL")) {
     info <- calc_MI_FB(
       matrix(current_theta$posterior_sample),
       item_parameter_sample, item_pool@NCAT, model_code)[, 1]
     return(info)
   }
+  if (item_method == "RANDOM") {
+    random <- runif(item_pool@ni, min = 0, max = 1)
+    return(random)
+  }
 }
 
-#' @noRd
+#' (Internal) Initialize groupings record
+#'
+#' \code{\link{initializeCompletedGroupingsRecord}} is an internal function for
+#' creating a new groupings record. Used for keeping track of completed groups such as item sets.
+#'
+#' @returns \code{\link{initializeCompletedGroupingsRecord}} returns a groupings record.
+#'
+#' @keywords internal
 initializeCompletedGroupingsRecord <- function() {
   o <- list()
   o$completed_stimulus_index <- NULL
