@@ -12,6 +12,7 @@ setClass("config_Shadow",
     exclude_policy     = "list",
     refresh_policy     = "list",
     exposure_control   = "list",
+    overlap_control    = "list",
     stopping_criterion = "list",
     interim_theta      = "list",
     final_theta        = "list",
@@ -45,7 +46,7 @@ setClass("config_Shadow",
     ),
     exclude_policy = list(
       method = "HARD",
-      M = NULL
+      M = 1000
     ),
     refresh_policy = list(
       method                    = "ALWAYS",
@@ -65,6 +66,11 @@ setClass("config_Shadow",
       fading_factor             = 0.999,
       diagnostic_stats          = FALSE
     ),
+    overlap_control = list(
+      method                    = "NONE",
+      M                         = 100,
+      max_overlap_rate          = 0.20
+    ),
     stopping_criterion = list(
       method                    = "FIXED",
       test_length               = NULL,
@@ -108,14 +114,17 @@ setClass("config_Shadow",
     theta_grid                  = seq(-4, 4, .1)
   ),
   validity = function(object) {
+
     err <- NULL
-    if (!toupper(object@MIP$solver) %in% c("RSYMPHONY", "LPSOLVE", "GUROBI", "RGLPK")) {
-      msg <- sprintf("config@MIP: unrecognized $solver '%s' (accepts RSYMPHONY, LPSOLVE, GUROBI, RGLPK)", object@MIP$solver)
+    if (!object@MIP$solver %in% c("RSYMPHONY", "HIGHS", "GUROBI", "LPSOLVE", "RGLPK")) {
+      # only capitalized names are valid values;
+      # the rest of the package assumes this is capitalized
+      msg <- sprintf("config@MIP: unrecognized $solver '%s' (accepts RSYMPHONY, HIGHS, GUROBI, LPSOLVE, or RGLPK)", object@MIP$solver)
       err <- c(err, msg)
     }
 
-    for (solver_name in c("gurobi", "Rsymphony", "Rglpk")) {
-      if (toupper(object@MIP$solver) == toupper(solver_name)) {
+    for (solver_name in c("Rsymphony", "highs", "gurobi", "Rglpk")) {
+      if (object@MIP$solver == toupper(solver_name)) {
         if (!requireNamespace(solver_name, quietly = TRUE)) {
           msg <- sprintf("config@MIP: could not find the specified solver package '%s'", solver_name)
           err <- c(err, msg)
@@ -123,8 +132,8 @@ setClass("config_Shadow",
       }
     }
 
-    if (!toupper(object@item_selection$method) %in% c("MFI", "MPWI", "EB", "FB", "GFI", "FIXED")) {
-      msg <- sprintf("config@item_selection: unrecognized $method '%s' (accepts MFI, MPWI, EB, FB, GFI, or FIXED)", object@item_selection$method)
+    if (!toupper(object@item_selection$method) %in% c("MFI", "MPWI", "EB", "FB", "GFI", "FIXED", "RANDOM")) {
+      msg <- sprintf("config@item_selection: unrecognized $method '%s' (accepts MFI, MPWI, EB, FB, GFI, FIXED or RANDOM)", object@item_selection$method)
       err <- c(err, msg)
     }
     if (toupper(object@item_selection$method) %in% c("FIXED")) {
@@ -158,7 +167,7 @@ setClass("config_Shadow",
       }
     }
     if (!object@refresh_policy$method %in%
-      c("ALWAYS", "POSITION", "INTERVAL", "THRESHOLD", "INTERVAL-THRESHOLD", "STIMULUS", "SET", "PASSAGE")) {
+      c("ALWAYS", "NONE", "POSITION", "INTERVAL", "THRESHOLD", "INTERVAL-THRESHOLD", "STIMULUS", "SET", "PASSAGE")) {
       msg <- sprintf("config@refresh_policy: unrecognized $method '%s'", object@refresh_policy$method)
       err <- c(err, msg)
     }
@@ -166,6 +175,10 @@ setClass("config_Shadow",
       msg <- sprintf("config@exposure_control: unrecognized $method '%s' (accepts NONE, ELIGIBILITY, BIGM, or BIGM-BAYESIAN)", object@exposure_control$method)
       err <- c(err, msg)
     }
+    if (!object@overlap_control$method %in% c("NONE", "ELIGIBILITY", "BIGM", "BIGM-BAYESIAN")) {
+      msg <- sprintf("config@overlap_control: unrecognized $method '%s' (accepts NONE, BIGM, or BIGM-BAYESIAN)", object@overlap_control$method)
+      err <- c(err, msg)
+    }
     if (object@exposure_control$method %in% c("BIGM", "BIGM-BAYESIAN")) {
       if (!is.numeric(object@exposure_control$M)) {
         if (!is.null(object@exposure_control$M)) {
@@ -180,6 +193,26 @@ setClass("config_Shadow",
         }
       }
     }
+    if (object@overlap_control$method %in% c("BIGM", "BIGM-BAYESIAN")) {
+      if (!is.numeric(object@overlap_control$M)) {
+        if (!is.null(object@overlap_control$M)) {
+          msg <- sprintf("$method 'BIGM', 'BIGM-BAYESIAM' requires $M to be a positive value")
+          err <- c(err, msg)
+        }
+      }
+      if (is.numeric(object@overlap_control$M)) {
+        if (object@overlap_control$M < 0) {
+          msg <- sprintf("$method 'BIGM', 'BIGM-BAYESIAM' requires $M to be a positive value")
+          err <- c(err, msg)
+        }
+      }
+    }
+    if (object@overlap_control$method == "ELIGIBILITY") {
+      if (object@exposure_control$method != "ELIGIBILITY") {
+        msg <- sprintf("config@overlap_control: $method 'ELIGIBILITY' also requires @exposure_control$method 'ELIGIBILITY'")
+        err <- c(err, msg)
+      }
+    }
     if (toupper(object@item_selection$method) %in% c("GFI") &
       object@exposure_control$method %in% c("ELIGIBILITY")) {
       msg <- sprintf("config@exposure_control: $method 'ELIGIBILITY' is not compatible with @item_selection$method 'GFI'")
@@ -197,20 +230,32 @@ setClass("config_Shadow",
       msg <- sprintf("config@exposure_control: unexpected length($max_exposure_rate) %s (must be 1 or $n_segment)", length(object@exposure_control$max_exposure_rate))
       err <- c(err, msg)
     }
+    if (!is.null(object@exposure_control$first_segment)) {
+      if (any(
+        !(object@exposure_control$first_segment %in% 1:object@exposure_control$n_segment)
+      )) {
+        msg <- sprintf(
+          "config@exposure_control: out-of-range values in $first_segment %s (must be within 1:n_segment; n_segment is %s)",
+          setdiff(object@exposure_control$first_segment, 1:object@exposure_control$n_segment),
+          object@exposure_control$n_segment
+        )
+        err <- c(err, msg)
+      }
+    }
     if (!object@stopping_criterion$method %in% c("FIXED")) {
       msg <- sprintf("config@stopping_criterion: unrecognized $method '%s'", object@stopping_criterion$method)
       err <- c(err, msg)
     }
-    if (!object@interim_theta$method %in% c("EAP", "MLE", "MLEF", "EB", "FB")) {
-      msg <- sprintf("config@interim_theta: unrecognized $method '%s' (accepts EAP, MLE, MLEF, EB, or FB)", object@interim_theta$method)
+    if (!object@interim_theta$method %in% c("EAP", "MLE", "MLEF", "EB", "FB", "CARRYOVER")) {
+      msg <- sprintf("config@interim_theta: unrecognized $method '%s' (accepts EAP, MLE, MLEF, EB, FB, or CARRYOVER)", object@interim_theta$method)
       err <- c(err, msg)
     }
     if (!object@interim_theta$prior_dist %in% c("NORMAL", "UNIFORM")) {
       msg <- sprintf("config@interim_theta: unrecognized $prior_dist '%s' (accepts NORMAL or UNIFORM)", object@interim_theta$prior_dist)
       err <- c(err, msg)
     }
-    if (!object@final_theta$method %in% c("EAP", "MLE", "MLEF", "EB", "FB")) {
-      msg <- sprintf("config@final_theta: unrecognized $method '%s' (accepts EAP, MLE, MLEF, EB, or FB)", object@final_theta$method)
+    if (!object@final_theta$method %in% c("EAP", "MLE", "MLEF", "EB", "FB", "CARRYOVER")) {
+      msg <- sprintf("config@final_theta: unrecognized $method '%s' (accepts EAP, MLE, MLEF, EB, FB, or CARRYOVER)", object@final_theta$method)
       err <- c(err, msg)
     }
     if (toupper(object@final_theta$method) == "EAP") {
@@ -223,6 +268,10 @@ setClass("config_Shadow",
       (!object@interim_theta$method %in% c("EB", "FB"))) {
       err <- c(err, "config@exposure_control: $method 'BIGM-BAYESIAN' requires interim_theta$method to be EB or FB")
     }
+    if ((object@overlap_control$method == c("BIGM-BAYESIAN")) &&
+        (!object@interim_theta$method %in% c("EB", "FB"))) {
+      err <- c(err, "config@overlap_control: $method 'BIGM-BAYESIAN' requires interim_theta$method to be EB or FB")
+    }
     if (length(err) == 0) {
       return(TRUE)
     } else {
@@ -233,7 +282,7 @@ setClass("config_Shadow",
 
 #' Create a config_Shadow object
 #'
-#' \code{\link{createShadowTestConfig}} is a config function for creating a \code{\linkS4class{config_Shadow}} object for Shadow test assembly.
+#' \code{\link{createShadowTestConfig}} is a config function for creating a \code{\linkS4class{config_Shadow}} object for shadowtest assembly.
 #' Default values are used for any unspecified parameters/slots.
 #'
 #' @param item_selection a named list containing item selection criteria.
@@ -250,7 +299,7 @@ setClass("config_Shadow",
 #' }
 #' @param MIP a named list containing solver options.
 #' \itemize{
-#'   \item{\code{solver}} the type of solver. Accepts \code{Rsymphony, gurobi, lpSolve, Rglpk}. (default = \code{LPSOLVE})
+#'   \item{\code{solver}} the type of solver. Accepts \code{Rsymphony, highs, gurobi, lpSolve, Rglpk}. (default = \code{LPSOLVE})
 #'   \item{\code{verbosity}} verbosity level of the solver. (default = \code{-2})
 #'   \item{\code{time_limit}} time limit in seconds. Used in solvers \code{Rsymphony, gurobi, Rglpk}. (default = \code{60})
 #'   \item{\code{gap_limit}} search termination criterion. Gap limit in relative scale passed onto the solver. Used in solver \code{gurobi}. (default = \code{.05})
@@ -270,7 +319,7 @@ setClass("config_Shadow",
 #'   \item{\code{method}} the type of policy. Accepts \code{HARD, SOFT}. (default = \code{HARD})
 #'   \item{\code{M}} the Big M penalty to use on item information. Used in the \code{SOFT} method.
 #' }
-#' @param refresh_policy a named list containing the refresh policy for when to obtain a new shadow test.
+#' @param refresh_policy a named list containing the refresh policy for when to obtain a new shadowtest.
 #' \itemize{
 #'   \item{\code{method}} the type of policy. Accepts \code{ALWAYS, POSITION, INTERVAL, THRESHOLD, INTERVAL-THRESHOLD, STIMULUS, SET, PASSAGE}. (default = \code{ALWAYS})
 #'   \item{\code{interval}} used in methods \code{INTERVAL, INTERVAL-THRESHOLD}. Set to 1 to refresh at each position, 2 to refresh at every two positions, and so on. (default = \code{1})
@@ -290,6 +339,12 @@ setClass("config_Shadow",
 #'   \item{\code{fading_factor}} the fading factor to apply. (default = \code{.999})
 #'   \item{\code{diagnostic_stats}} set to \code{TRUE} to generate segment-wise diagnostic statistics. (default = \code{FALSE})
 #' }
+#' @param overlap_control a named list containing overlap control settings.
+#' \itemize{
+#'   \item{\code{method}} the type of overlap control method. Accepts \code{NONE, ELIGIBILITY, BIGM, BIGM-BAYESIAN}. (default = \code{NONE})
+#'   \item{\code{M}} used in methods \code{BIGM, BIGM-BAYESIAN}. the Big M penalty to use on item information.
+#'   \item{\code{max_overlap_rate}} target overlap rate. (default = \code{0.20})
+#' }
 #' @param stopping_criterion a named list containing stopping criterion.
 #' \itemize{
 #'   \item{\code{method}} the type of stopping criterion. Accepts \code{FIXED}. (default = \code{FIXED})
@@ -300,7 +355,7 @@ setClass("config_Shadow",
 #' }
 #' @param interim_theta a named list containing interim theta estimation options.
 #' \itemize{
-#'   \item{\code{method}} the type of estimation. Accepts \code{EAP, MLE, MLEF, EB, FB}. (default = \code{EAP})
+#'   \item{\code{method}} the type of estimation. Accepts \code{EAP, MLE, MLEF, EB, FB, CARRYOVER}. (default = \code{EAP})
 #'   \item{\code{shrinkage_correction}} set \code{TRUE} to apply shrinkage correction. Used when \code{method} is \code{EAP}. (default = \code{FALSE})
 #'   \item{\code{prior_dist}} the type of prior distribution. Accepts \code{NORMAL, UNIFORM}. (default = \code{NORMAL})
 #'   \item{\code{prior_par}} distribution parameters for \code{prior_dist}. (default = \code{c(0, 1)})
@@ -318,7 +373,7 @@ setClass("config_Shadow",
 #' }
 #' @param final_theta a named list containing final theta estimation options.
 #' \itemize{
-#'   \item{\code{method}} the type of estimation. Accepts \code{EAP, MLE, MLEF, EB, FB}. (default = \code{EAP})
+#'   \item{\code{method}} the type of estimation. Accepts \code{EAP, MLE, MLEF, EB, FB, CARRYOVER}. (default = \code{EAP})
 #'   \item{\code{shrinkage_correction}} set \code{TRUE} to apply shrinkage correction. Used when \code{method} is \code{EAP}. (default = \code{FALSE})
 #'   \item{\code{prior_dist}} the type of prior distribution. Accepts \code{NORMAL, UNIFORM}. (default = \code{NORMAL})
 #'   \item{\code{prior_par}} distribution parameters for \code{prior_dist}. (default = \code{c(0, 1)})
@@ -347,14 +402,14 @@ setClass("config_Shadow",
 #' @export
 createShadowTestConfig <- function(
   item_selection = NULL, content_balancing = NULL, MIP = NULL, MCMC = NULL,
-  exclude_policy = NULL, refresh_policy = NULL, exposure_control = NULL, stopping_criterion = NULL,
-  interim_theta = NULL, final_theta = NULL, theta_grid = seq(-4, 4, .1)) {
+  exclude_policy = NULL, refresh_policy = NULL, exposure_control = NULL, overlap_control = NULL,
+  stopping_criterion = NULL, interim_theta = NULL, final_theta = NULL, theta_grid = seq(-4, 4, .1)) {
   cfg <- new("config_Shadow")
 
   arg_names <- c(
     "item_selection", "content_balancing", "MIP", "MCMC",
-    "exclude_policy", "refresh_policy", "exposure_control", "stopping_criterion",
-    "interim_theta", "final_theta"
+    "exclude_policy", "refresh_policy", "exposure_control", "overlap_control",
+    "stopping_criterion", "interim_theta", "final_theta"
   )
   obj_names <- c()
   for (arg in arg_names) {
@@ -385,6 +440,8 @@ createShadowTestConfig <- function(
     }
   }
 
+  # ensure the solver name is capitalized here;
+  # the rest of the package assumes this is already done
   cfg@MIP$solver <- toupper(cfg@MIP$solver)
 
   if (!is.null(theta_grid)) {
@@ -414,11 +471,13 @@ createShadowTestConfig <- function(
 #'   }}
 #' }
 #'
+#' @slot call the function call used for obtaining this object.
 #' @slot output a length-*nj* list of \code{\linkS4class{output_Shadow}} objects, containing the assembly results for each participant.
 #' @slot final_theta_est a length-*nj* vector containing final theta estimates for each participant.
 #' @slot final_se_est a length-*nj* vector standard errors of the final theta estimates for each participant.
 #' @slot exposure_rate a matrix containing item-level exposure rates of all items in the pool. Also contains stimulus-level exposure rates if the assembly was set-based.
 #' @slot usage_matrix a *nj* by (*ni* + *ns*) matrix representing whether the item/stimulus was administered to each participant. Stimuli representations are appended to the right side of the matrix.
+#' @slot cumulative_usage_matrix a *nj* by (*ni* + *ns*) matrix representing the number of times the item/stimulus was administered to each participant over multiple administrations.
 #' @slot true_segment_count a length-*nj* vector containing the how many examinees are now in their segment based on the true theta. This will tend to increase. This can be reproduced with true theta values alone.
 #' @slot est_segment_count a length-*nj* vector containing the how many examinees are now in their segment based on the estimated theta. This will tend to increase. This can be reproduced with estimated theta values alone.
 #' @slot eligibility_stats exposure record for diagnostics.
@@ -432,15 +491,19 @@ createShadowTestConfig <- function(
 #' @slot data the \code{data} argument used in the assembly.
 #' @slot prior the \code{prior} argument used in the assembly.
 #' @slot prior_par the \code{prior_par} argument used in the assembly.
+#' @slot adaptivity a list of adaptivity indices.
+#' @slot simulation_constants a list containing simulation constants parsed from input.
 #'
 #' @export
 setClass("output_Shadow_all",
   slots = c(
+    call                        = "call",
     output                      = "list_or_null",
     final_theta_est             = "matrix_or_numeric_or_null",
     final_se_est                = "matrix_or_numeric_or_null",
     exposure_rate               = "matrix_or_null",
     usage_matrix                = "matrix_or_null",
+    cumulative_usage_matrix     = "matrix_or_null",
     true_segment_count          = "numeric_or_null",
     est_segment_count           = "numeric_or_null",
     eligibility_stats           = "list_or_null",
@@ -453,7 +516,9 @@ setClass("output_Shadow_all",
     data                        = "matrix_or_null",
     true_theta                  = "matrix_or_numeric_or_null",
     prior                       = "matrix_or_numeric_or_null",
-    prior_par                   = "matrix_or_numeric_or_null"
+    prior_par                   = "matrix_or_numeric_or_null",
+    adaptivity                  = "list_or_null",
+    simulation_constants        = "list_or_null"
   ),
   prototype = list(
     output                      = NULL,
@@ -461,6 +526,7 @@ setClass("output_Shadow_all",
     final_se_est                = NULL,
     exposure_rate               = NULL,
     usage_matrix                = NULL,
+    cumulative_usage_matrix     = NULL,
     true_segment_count          = NULL,
     est_segment_count           = NULL,
     eligibility_stats           = NULL,
@@ -473,7 +539,9 @@ setClass("output_Shadow_all",
     data                        = NULL,
     true_theta                  = NULL,
     prior                       = NULL,
-    prior_par                   = NULL
+    prior_par                   = NULL,
+    adaptivity                  = NULL,
+    simulation_constants        = NULL
   ),
   validity = function(object) {
     return(TRUE)
@@ -493,7 +561,7 @@ setClass("output_Shadow_all",
 #' @slot administered_item_resp item responses from the simulee at each position.
 #' @slot administered_item_ncat the number of categories of each administered item.
 #' @slot administered_stimulus_index stimulus IDs administered at each position.
-#' @slot shadow_test_refreshed \code{TRUE} indicates the shadow test was refreshed for the position.
+#' @slot shadow_test_refreshed \code{TRUE} indicates the shadowtest was refreshed for the position.
 #' @slot shadow_test_feasible \code{TRUE} indicates the MIP was feasible with all constraints.
 #' @slot solve_time elapsed time in running the solver at each position.
 #' @slot initial_theta_est initial theta estimate.
@@ -505,7 +573,7 @@ setClass("output_Shadow_all",
 #' @slot posterior the posterior distribution after completing test.
 #' @slot posterior_sample posterior samples of interim theta before the estimation of final theta. \code{mean(posterior_sample) == interim_theta_est[test_length]} holds.
 #' @slot likelihood the likelihood distribution after completing test.
-#' @slot shadow_test the list containing the item IDs within the shadow test used in each position.
+#' @slot shadow_test the list containing the item IDs within the shadowtest used in each position.
 #' @slot max_cat_pool the maximum number of response categories the item pool had.
 #' @slot ni_pool the total number of items the item pool had.
 #' @slot ns_pool the total number of stimuli the item pool had.
